#ifndef TV_H
#define TV_H

#include <string>
using namespace std;

#include <pthread.h>

#include "NuppelVideoRecorder.h"
#include "NuppelVideoPlayer.h"
#include "RingBuffer.h"
#include "settings.h"

#include "channel.h"
#include "recordinginfo.h"

typedef enum 
{
    kState_Error = -1,
    kState_None = 0,
    kState_WatchingLiveTV,
    kState_WatchingPreRecorded,
    kState_WatchingRecording,       // watching _what_ you're recording
    kState_WatchingOtherRecording,  // watching something else
    kState_RecordingOnly
} TVState;

class QSqlDatabase;

class TV
{
 public:
    TV(const string &startchannel);
   ~TV(void);

    void LiveTV(void);

    int AllowRecording(RecordingInfo *rcinfo, int timeuntil);
    void StartRecording(RecordingInfo *rcinfo);
    void StopRecording(void);
    RecordingInfo *GetRecording(void) { return curRecording; }

    // next two functions only work on recorded programs.
    void Playback(RecordingInfo *rcinfo);
    char *GetScreenGrab(RecordingInfo *rcinfo, int secondsin, int &bufferlen,
                        int &video_width, int &video_height);

    bool IsRunning(void) { return runMainLoop; }
    void Stop(void) { runMainLoop = false; }

    TVState GetState(void) { return internalState; }

    bool CheckChannel(char *channum); 
    bool ChangeExternalChannel(char *channum);

    string GetFilePrefix() { return settings->GetSetting("RecordFilePrefix"); }
 protected:
    void doLoadMenu(void);
    static void *MenuHandler(void *param);

    void RunTV(void);
    static void *EventThread(void *param);

 private:
    void ChangeChannel(bool up);
    void ChangeChannel(char *name);
    
    void ChannelKey(int key);
    void ChannelCommit(void);

    void ToggleInputs(void); 
 
    void UpdateOSD(void); 
    void GetChannelInfo(int lchannel, string &title, string &subtitle, 
                        string &desc, string &category, string &starttime,
                        string &endtime);

    void ConnectDB(void);
    void DisconnectDB(void);

    void LoadMenu(void);

    void SetupRecorder();
    void TeardownRecorder(bool killFile = false);

    void SetupPlayer();
    void TeardownPlayer();

    void ProcessKeypress(int keypressed);

    void StateToString(TVState state, string &statestr);
    void HandleStateChange();
    bool StateIsRecording(TVState state);
    bool StateIsPlaying(TVState state);
    TVState RemovePlaying(TVState state);
    TVState RemoveRecording(TVState state);

    void WriteRecordedRecord();

    NuppelVideoRecorder *nvr;
    NuppelVideoPlayer *nvp;

    RingBuffer *rbuffer, *prbuffer;
    Channel *channel;

    Settings *settings;

    int osd_display_time;

    bool channelqueued;
    char channelKeys[4];
    int channelkeysstored;

    QSqlDatabase *db_conn;

    bool menurunning;

    TVState internalState;

    bool runMainLoop;
    bool exitPlayer;
    bool paused;
    time_t recordEndTime;

    float frameRate;

    pthread_t event, encode, decode;
    bool changeState;
    TVState nextState;
    string inputFilename, outputFilename;

    bool watchingLiveTV;

    RecordingInfo *curRecording;
    int tvtorecording;
};

#endif
